package be.iminds.ilabt.jfed.rspec.model;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * HardwareTypeInfo: The hardwaretype info extension.
 *
 * Note: this is completely immutable
 *
 * Example: <rspec xmlns="http://www.geni.net/resources/rspec/3" xmlns:hwinfo="https://doc.fed4fire.eu/rspec/ext/hwinfo/1" >
 *   <hwinfo:hardware_type_info>
 *     <hwinfo:overview media-type="text/plain">Short textual overview</overview>
 *     <hwinfo:overview media-type="text/html" href="https://doc.ilabt.imec.be/ilabt-documentation/virtualwallfacility.html#virtual-wall-2"/>
 *
 *     <hwinfo:hardware_type name="pcgen05" hrn="Generation 5">
 *        <hwinfo:info media-type="text/plain">
 *          1x 4 core E3-1220v3 (3.1GHz), 16GB RAM, 1x 250GB harddisk, 2-10 gigabit connections
 *        </hwinfo:info>
 *        <hwinfo:info media-type="text/html" href="https://doc.ilabt.imec.be/ilabt-documentation/virtualwallfacility.html#pcgen5"/>
 *        <hwinfo:info media-type="application/json" href="http://www.virtual.wall/api/pcgen05.json"/>
 *        <hwinfo:info media-type="application/xml" href="http://www.virtual.wall/api/pcgen05.xml"/>
 *     </hwinfo:hardware_type>
 *     <hwinfo:hardware_type name="pcgen04" hrn="Generation 4">
 *        ...
 *     </hwinfo:hardware_type>
 *        ...
 *   </hwinfo:hardware_type_info>
 *   ...
 * </rspec>
 *
 */
public class HardwareTypeInfo {
    public static class Entry {
        @Nullable public final String mediaType;
        @Nullable public final String href;
        @Nullable public final String directValue;

        public Entry(@Nullable String mediaType, @Nullable String href, @Nullable String directValue) {
            this.mediaType = mediaType;
            this.href = href;
            this.directValue = directValue;
        }

        @Nullable
        public String getMediaType() {
            return mediaType;
        }

        @Nullable
        public String getHref() {
            return href;
        }

        @Nullable
        public String getDirectValue() {
            return directValue;
        }
    }

    public static class HardwareTypeInfoEntry {
        @Nullable public final String name;
        @Nullable public final String hrn;
        @Nonnull public final List<Entry> infos;

        public HardwareTypeInfoEntry(@Nullable String name,
                                     @Nullable String hrn,
                                     @Nullable List<Entry> infos) {
            this.name = name;
            this.hrn = hrn;
            this.infos = infos == null ? Collections.emptyList() : Collections.unmodifiableList(new ArrayList<>(infos));
        }

        @Nullable
        public String getName() {
            return name;
        }

        @Nullable
        public String getHrn() {
            return hrn;
        }

        @Nullable
        public String getTextDescription() {
            return infos.stream()
                    .filter(e -> Objects.equals(e.getMediaType(), "text/plain"))
                    .map(Entry::getDirectValue)
                    .findFirst().orElse(null);
        }

        @Nullable
        public String getHtmlDescription() {
            return infos.stream()
                    .filter(e -> Objects.equals(e.getMediaType(), "text/html"))
                    .map(Entry::getHref)
                    .findFirst().orElse(null);
        }

        @Nonnull
        public List<Entry> getInfos() {
            return infos;
        }
    }

    @Nonnull public final List<Entry> overview;
    @Nonnull public final List<HardwareTypeInfoEntry> hardwareTypeInfoEntries;

    public HardwareTypeInfo(@Nullable List<Entry> overview,
                            @Nullable List<HardwareTypeInfoEntry> hardwareTypeInfoEntries) {
        this.overview = overview == null ? Collections.emptyList() : Collections.unmodifiableList(new ArrayList<>(overview));
        this.hardwareTypeInfoEntries = hardwareTypeInfoEntries == null ? Collections.emptyList() : Collections.unmodifiableList(new ArrayList<>(hardwareTypeInfoEntries));
    }

    @Nullable
    public String getTextOverview() {
        return overview.stream()
                .filter(e -> Objects.equals(e.getMediaType(), "text/plain"))
                .map(Entry::getDirectValue)
                .findFirst().orElse(null);
    }

    @Nullable
    public String getHtmlOverview() {
        return overview.stream()
                .filter(e -> Objects.equals(e.getMediaType(), "text/html"))
                .map(Entry::getHref)
                .findFirst().orElse(null);
    }

    @Nonnull
    public List<Entry> getOverviews() {
        return overview;
    }

    @Nonnull
    public List<HardwareTypeInfoEntry> getHardwareTypeInfoEntries() {
        return hardwareTypeInfoEntries;
    }

    @Nullable
    public HardwareTypeInfoEntry getHardwareTypeInfoByName(@Nonnull String name) {
        return hardwareTypeInfoEntries.stream()
                .filter(e -> Objects.equals(e.getName(), name))
                .findFirst().orElse(null);
    }
}
